searchData={"items":[{"type":"module","title":"Profiler","doc":"This package contains a couple of profiling shell scripts to aid\nlive-system investigation. Among them:\n\n\n1) Easy performance visualization using kcachegrind\n2) Shell information using a sampling provider\n3) Memory information\n\nThis sampling profiler is intendend for shell and remote shell usage.\nMost commands here print their results to the screen for human inspection.\n\nExample usage:\n```\niex(2)> Profiler.profile(\"<0.187.0>\")\n100% {:proc_lib, :init_p_do_apply, 3, [file: 'proc_lib.erl', line: 249]}\n  100% {IEx.Evaluator, :init, 4, [file: 'lib/iex/evaluator.ex', line: 27]}\n    100% {IEx.Evaluator, :loop, 1, [file: 'lib/iex/evaluator.ex', line: 103]}\n        100% {IEx.Evaluator, :eval, 3, [file: 'lib/iex/evaluator.ex', line: 217]}\n          100% {IEx.Evaluator, :do_eval, 3, [file: 'lib/iex/evaluator.ex', line: 239]}\n              100% {IEx.Evaluator, :handle_eval, 5, [file: 'lib/iex/evaluator.ex', line: 258]}\n                100% {:elixir, :eval_forms, 3, [file: 'src/elixir.erl', line: 263]}\n                    100% {:elixir, :recur_eval, 3, [file: 'src/elixir.erl', line: 278]}\n                      100% {:erl_eval, :do_apply, 6, [file: 'erl_eval.erl', line: 680]}\n                          100% {Profiler, :profile, 2, [file: 'lib/profiler.ex', line: 120]}\n                            100% {Enum, :reduce_range_inc, 4, [file: 'lib/enum.ex', line: 3371]}\n                                100% {Profiler, :\"-profile/2-fun-0-\", 3, [file: 'lib/profiler.ex', line: 121]}\n```","ref":"Profiler.html"},{"type":"function","title":"Profiler.cancel_warn_if_stuck/1","doc":"","ref":"Profiler.html#cancel_warn_if_stuck/1"},{"type":"function","title":"Profiler.convert/1","doc":"","ref":"Profiler.html#convert/1"},{"type":"function","title":"Profiler.demo_fib/1","doc":"Demo function for the documentation.\n  Never implement fibonacci like this. Never","ref":"Profiler.html#demo_fib/1"},{"type":"function","title":"Profiler.format_stacktrace/2","doc":"","ref":"Profiler.html#format_stacktrace/2"},{"type":"function","title":"Profiler.fprof/2","doc":"This runs fprof the given amount of milliseconds or 5 seconds by default.\n\n  When the run completes the code tries to open kcachegrind to show the resultung\n  kcachegrind file. Ensure to have kcachegrind installed. If kcachgrind is not found\n  the function will just return the name of the generated report file. It\n  can then be copied to another lcoation for analysis\n\n  For pid there are five different input formats allowed:\n\n  1. fun() which will then be spwaned called in a loop and killed after the test\n  2. Native pid()\n  3. An atom that is resolved using whereis(name)\n  4. A string of the format \" \" or \"0.b.c\" or just \"b\" in which\n     case the pid is interpreted as \"<0.b.0>\"\n  5. An integer, in which case the pid is interpreted as \"<0.#{int}.0>\"\n\n  In this example the profiler is used to profile itself. The first percentage\n  number shows how many samples were found in the given function call.\n  Indention indicates the call stack:\n  ```\n  iex(1)> Profiler.fprof(fn -> Profiler.demo_fib(30) end)\n  ```","ref":"Profiler.html#fprof/2"},{"type":"function","title":"Profiler.hotspots/1","doc":"hotspots returns process names / stacktraces by the amount they are seen in the\n  given the timeout.","ref":"Profiler.html#hotspots/1"},{"type":"function","title":"Profiler.memory_usage/0","doc":"","ref":"Profiler.html#memory_usage/0"},{"type":"function","title":"Profiler.pid/1","doc":"Converts a term to a pid.\nAccepts\n- pid()\n- integer()\n- tuple()\n- atom()\n- binary()\n- string()\n\nExamples:\n```\niex(1)> Profiler.pid(1)\n<0.1.0>\niex(2)> Profiler.pid(\"1\")\n<0.1.0>\niex(3)> Profiler.pid(\"Elixir.IEx.Evaluator\")\n<0.187.0>\niex(4)> Profiler.pid({:IEx.Evaluator, :init, 4})\n```","ref":"Profiler.html#pid/1"},{"type":"function","title":"Profiler.print_hotspots/1","doc":"print_hotspots lists process names / stacktraces by the amount they are seen in the\n  given the timeout.","ref":"Profiler.html#print_hotspots/1"},{"type":"function","title":"Profiler.print_stacktrace/2","doc":"","ref":"Profiler.html#print_stacktrace/2"},{"type":"function","title":"Profiler.process_name/1","doc":"Identify the best human readable name for a process. Supports as\narguments all types supported by `pid/1`\n\nExamples:\n```\niex(1)> Profiler.process_name(1)\n:erst_code_purger\niex(2)> Profiler.process_name(self())\n{IEx.Evaluator, :init, 5}\n```","ref":"Profiler.html#process_name/1"},{"type":"function","title":"Profiler.processes/1","doc":"Processes lists all processes ordered by reductions withing the given\n  timeout. For that it takes an initial snapshot, sleeps the given timeout\n  and takes a second snapshot.\n\n  The following options are supported:\n\n  - :timeout - the timeout in milliseconds\n  - :limit - the limit of processes to list\n  - :stacktrace - the number of processes to print the stack trace for\n  - :profile - the number of processes to profile\n  ```\n  iex(1)> Profiler.processes\n  [<0.187.0>,{'Elixir.IEx.Evaluator',init,4},1339]\n  [<0.132.0>,tls_client_ticket_store,32]\n  [<0.182.0>,{'Elixir.Logger.Watcher',init,1},1]\n  [<0.181.0>,'Elixir.Logger.BackendSupervisor',1]\n  [<0.180.0>,{'Elixir.Logger.Watcher',init,1},1]\n  [<0.179.0>,'Elixir.Logger',1]\n  [<0.178.0>,'Elixir.Logger.Supervisor',1]\n  [<0.177.0>,{application_master,start_it,4},1]\n  [<0.176.0>,{application_master,init,4},1]\n  [<0.161.0>,'Elixir.Hex.UpdateChecker',1]\n  :ok\n  ```","ref":"Profiler.html#processes/1"},{"type":"function","title":"Profiler.processes_memory/0","doc":"Processes lists all processes ordered by heap usage.","ref":"Profiler.html#processes_memory/0"},{"type":"function","title":"Profiler.processes_message_queue_len/0","doc":"Lists the processes by message queue length.","ref":"Profiler.html#processes_message_queue_len/0"},{"type":"function","title":"Profiler.profile/2","doc":"This runs the sampling profiler for the given amount of milliseconds or\n  10 seconds by default. The sampling profiler will collect stack traces\n  of the given process pid or process name and print the collected samples\n  based on frequency.\n\n  For pid there are five different input formats allowed:\n\n  1. fun() which will then be spwaned called in a loop and killed after the test\n  2. Native pid()\n  3. An atom that is resolved using whereis(name)\n  4. A string of the format \" \" or \"0.b.c\" or just \"b\" in which\n     case the pid is interpreted as \"<0.b.0>\"\n  5. An integer, in which case the pid is interpreted as \"<0.#{int}.0>\"\n\n  In this example the profiler is used to profile itself. The first percentage\n  number shows how many samples were found in the given function call.\n  Indention indicates the call stack:\n  ```\n  iex(2)> Profiler.profile(187)\n  100% {:proc_lib, :init_p_do_apply, 3, [file: 'proc_lib.erl', line: 249]}\n    100% {IEx.Evaluator, :init, 4, [file: 'lib/iex/evaluator.ex', line: 27]}\n      100% {IEx.Evaluator, :loop, 1, [file: 'lib/iex/evaluator.ex', line: 103]}\n          100% {IEx.Evaluator, :eval, 3, [file: 'lib/iex/evaluator.ex', line: 217]}\n            100% {IEx.Evaluator, :do_eval, 3, [file: 'lib/iex/evaluator.ex', line: 239]}\n                100% {IEx.Evaluator, :handle_eval, 5, [file: 'lib/iex/evaluator.ex', line: 258]}\n                  100% {:elixir, :eval_forms, 3, [file: 'src/elixir.erl', line: 263]}\n                      100% {:elixir, :recur_eval, 3, [file: 'src/elixir.erl', line: 278]}\n                        100% {:erl_eval, :do_apply, 6, [file: 'erl_eval.erl', line: 680]}\n                            100% {Profiler, :profile, 2, [file: 'lib/profiler.ex', line: 120]}\n                              100% {Enum, :reduce_range_inc, 4, [file: 'lib/enum.ex', line: 3371]}\n                                  100% {Profiler, :\"-profile/2-fun-0-\", 3, [file: 'lib/profiler.ex', line: 121]}\n  ```","ref":"Profiler.html#profile/2"},{"type":"function","title":"Profiler.profile_simple/2","doc":"Arguments are the same as for profile() but this sampling profiler does not\n  analyze stacktrace but instead just samples the current function and prints\n  the result.\n\n  The first number shows the total number of samples that have been recorded\n  per function call.\n\n  For pid there are five different input formats allowed:\n\n  1. fun() which will then be spwaned called in a loop and killed after the test\n  2. Native pid()\n  3. An atom that is resolved using whereis(name)\n  4. A string of the format \" \" or \"0.b.c\" or just \"b\" in which\n     case the pid is interpreted as \"<0.b.0>\"\n  5. An integer, in which case the pid is interpreted as \"<0.#{int}.0>\"\n\n  ```\n  iex(2)> Profiler.profile_simple 197\n  {10000, {Profiler, :\"-profile_simple/2-fun-0-\", 3}}\n  ```","ref":"Profiler.html#profile_simple/2"},{"type":"function","title":"Profiler.stacktrace/2","doc":"Returns current stacktrace for the given pid and allows setting the erlang\n  internal stacktrace depth.","ref":"Profiler.html#stacktrace/2"},{"type":"function","title":"Profiler.time/2","doc":"Times the given function and prints the result.\n  Example usage:\n  ```\n  iex(1)> Profiler.time(fn() -> Process.sleep 1000 end)\n  timer: 1004\n  :ok\n  ```","ref":"Profiler.html#time/2"},{"type":"function","title":"Profiler.trace_all_calls/2","doc":"","ref":"Profiler.html#trace_all_calls/2"},{"type":"function","title":"Profiler.trace_poll_stacktrace/2","doc":"","ref":"Profiler.html#trace_poll_stacktrace/2"},{"type":"function","title":"Profiler.warn_if_stuck/2","doc":"In order to identify processes that are supposed to be short-lived but actually\ntake too mich time this function produces Logger.warning() with a stacktrace if\nthe provided process is still alive after the given timeout.\n\nIf the process is just running a certain critical section that should be monitored\nthe warning can be supressed using `cancel_warn_if_stuck()`\n\n\n\n@pid Pid of the process to be monitored, or a function after which the warning will be autocancelled\n@opts can be\n  - `timeout` -> Timeout in ms after which the warning will be produced. defaults to 5_000\n  - `label` -> Process label to be used in the report or just the `pid` will be used\n  - `fun` -> When provided this function replaces the default report via `Logger.warning`","ref":"Profiler.html#warn_if_stuck/2"},{"type":"type","title":"Profiler.task/0","doc":"","ref":"Profiler.html#t:task/0"},{"type":"extras","title":"Sampling Profiler","doc":"# Sampling Profiler\n\nThe sampling profiler is a single process profiler to inspect runtime of time consuming processes. This is especially useful for doing performance analysis and optimization.\n\nPlease checkout the full documentation at: https://hexdocs.pm/profiler/Profiler.html\n\nExample output for profiling the [Diode Blockchain Server](https://github.com/diodechain/diode_server_ex):\n```\niex(2)> Profiler.profile(Chain.Worker)\n 100% {:proc_lib, :init_p_do_apply, 3, [file: 'proc_lib.erl', line: 249]}\n    100% {:gen_server, :handle_msg, 6, [file: 'gen_server.erl', line: 711]}\n       100% {:gen_server, :try_dispatch, 4, [file: 'gen_server.erl', line: 637]}\n           26% {Chain.Worker, :handle_info, 2, [file: 'lib/chain/worker.ex', line: 103]}\n              26% {Process, :sleep, 1, [file: 'lib/process.ex', line: 253]}\n           74% {Chain.Worker, :handle_info, 2, [file: 'lib/chain/worker.ex', line: 110]}\n              74% {Chain.Worker, :do_work, 1, [file: 'lib/chain/worker.ex', line: 122]}\n                24% {Chain.Block, :sign, 2, [file: 'lib/chain/block.ex', line: 269]}\n                    24% {Chain.Header, :sign, 2, [file: 'lib/chain/header.ex', line: 59]}\n                      24% {Chain.Header, :encode_egg, 1, [file: 'lib/chain/header.ex', line: 31]}\n                50% {Chain.Block, :sign, 2, [file: 'lib/chain/block.ex', line: 270]}\n                    50% {Chain.Header, :update_hash, 1, [file: 'lib/chain/header.ex', line: 54]}\n                      50% {Chain.Header, :encode_chicken, 1, [file: 'lib/chain/header.ex', line: 43]}\n                          50% {MnesiaMerkleTree, :root_hash, 1, [file: 'lib/mnesiamerkletree.ex', line: 62]}\n                            49% {MnesiaMerkleTree, :update_merkle_hash_count, 1, [file: 'lib/mnesiamerkletree.ex', line: 393]}\n                                49% {:lists, :zipwith, 3, [file: 'lists.erl', line: 451]}\n                                  49% {:lists, :zipwith, 3, [file: 'lists.erl', line: 451]}\n                                      13% {MnesiaMerkleTree, :signature, 1, [file: 'lib/mnesiamerkletree.ex', line: 460]}\n                                      23% {MnesiaMerkleTree, :signature, 1, [file: 'lib/mnesiamerkletree.ex', line: 461]}\n                                        23% {BertExt, :encode!, 1, [file: 'lib/bert.ex', line: 43]}\n                                            12% {Enum, :\"-map/2-lists^map/1-0-\", 2, []}\n                                            9% {Enum, :map, 2, [file: 'lib/enum.ex', line: 1395]}\n:ok\n```\n\n# fprof Sugar\n\nAlso the profiler has tooling to profile using the erlang native `fprof` module and will call the external [kcachegrind](https://kcachegrind.github.io/html/Home.html) tool when available for visualization.\n\nMacOS: `brew install qcachegrind`\nUbuntu: `sudo apt install kcachegrind`\nWindows: https://sourceforge.net/projects/qcachegrindwin/files/latest/download\n\n```\niex(6)> Profiler.fprof(fn -> ExSha3Tiny.sha3_256(str) end)\n```\n\n![kcachegrind preview](https://raw.githubusercontent.com/dominicletz/profiler/master/images/kcachegrind.png)","ref":"readme.html"},{"type":"extras","title":"Installation - Sampling Profiler","doc":"The profiler can be installed by adding `profiler` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:profiler, \"~> 0.2\"}\n  ]\nend\n```","ref":"readme.html#installation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}